#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "citrix-server.h"
#include "defines.h"

static void citrix_server_class_init (CitrixServerClass *klass);
static void citrix_server_init       (CitrixServer *self);
static void citrix_server_dispose    (GObject *object);
static void citrix_server_finalize   (GObject *object);
static GVariant *  get_properties        (Server * server);

G_DEFINE_TYPE (CitrixServer, citrix_server, SERVER_TYPE);

static void
citrix_server_class_init (CitrixServerClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	object_class->dispose = citrix_server_dispose;
	object_class->finalize = citrix_server_finalize;

	ServerClass * server_class = SERVER_CLASS(klass);

	server_class->get_properties = get_properties;

	return;
}

static void
citrix_server_init (CitrixServer *self)
{

	return;
}

static void
citrix_server_dispose (GObject *object)
{

	G_OBJECT_CLASS (citrix_server_parent_class)->dispose (object);
	return;
}

static void
citrix_server_finalize (GObject *object)
{

	G_OBJECT_CLASS (citrix_server_parent_class)->finalize (object);
	return;
}

static GVariant *
get_properties (Server * server)
{
	GVariantBuilder propbuilder;
	g_variant_builder_init(&propbuilder, G_VARIANT_TYPE_ARRAY);

	GVariantBuilder namebuilder;
	g_variant_builder_init(&namebuilder, G_VARIANT_TYPE_TUPLE);
	g_variant_builder_add_value(&namebuilder, g_variant_new_string("username"));
	g_variant_builder_add_value(&namebuilder, g_variant_new_boolean(TRUE));
	g_variant_builder_add_value(&namebuilder, g_variant_new_variant(g_variant_new_string("")));
	g_variant_builder_add_value(&namebuilder, g_variant_new((const gchar *)G_VARIANT_TYPE_VARDICT));
	g_variant_builder_add_value(&propbuilder, g_variant_builder_end(&namebuilder));

	GVariantBuilder passbuilder;
	g_variant_builder_init(&passbuilder, G_VARIANT_TYPE_TUPLE);
	g_variant_builder_add_value(&passbuilder, g_variant_new_string("password"));
	g_variant_builder_add_value(&passbuilder, g_variant_new_boolean(TRUE));
	g_variant_builder_add_value(&passbuilder, g_variant_new_variant(g_variant_new_string("")));
	g_variant_builder_add_value(&passbuilder, g_variant_new((const gchar *)G_VARIANT_TYPE_VARDICT));
	g_variant_builder_add_value(&propbuilder, g_variant_builder_end(&passbuilder));

	GVariantBuilder domainbuilder;
	g_variant_builder_init(&domainbuilder, G_VARIANT_TYPE_TUPLE);
	g_variant_builder_add_value(&domainbuilder, g_variant_new_string("domain"));
	g_variant_builder_add_value(&domainbuilder, g_variant_new_boolean(TRUE));
	g_variant_builder_add_value(&domainbuilder, g_variant_new_variant(g_variant_new_string("")));
	g_variant_builder_add_value(&domainbuilder, g_variant_new((const gchar *)G_VARIANT_TYPE_VARDICT));
	g_variant_builder_add_value(&propbuilder, g_variant_builder_end(&domainbuilder));

	return g_variant_builder_end(&propbuilder);
}

Server *
citrix_server_new_from_keyfile (GKeyFile * keyfile, const gchar * name)
{
	g_return_val_if_fail(keyfile != NULL, NULL); /* NOTE: No way to check if that's really a keyfile :-( */
	g_return_val_if_fail(name != NULL, NULL);

	gchar * groupname = g_strdup_printf("%s %s", CONFIG_CITRIX_PREFIX, name);

	if (!g_key_file_has_group(keyfile, groupname)) {
		g_warning("Server '%s' specified but group '%s' was not found", name, groupname);
		g_free(groupname);
		return NULL;
	}

	CitrixServer * server = g_object_new(CITRIX_SERVER_TYPE, NULL);

	if (g_key_file_has_key(keyfile, groupname, "Name", NULL)) {
		server->parent.name = g_key_file_get_string(keyfile, groupname, CONFIG_SERVER_NAME, NULL);
	}

	if (g_key_file_has_key(keyfile, groupname, "URI", NULL)) {
		server->parent.uri = g_key_file_get_string(keyfile, groupname, CONFIG_SERVER_URI, NULL);
	}

	g_free(groupname);
	return SERVER(server);
}

/* Build the Citrix server from information in the JSON object */
Server *
citrix_server_new_from_json (JsonObject * object)
{
	CitrixServer * server = g_object_new(CITRIX_SERVER_TYPE, NULL);

	if (json_object_has_member(object, JSON_SERVER_NAME)) {
		JsonNode * node = json_object_get_member(object, JSON_SERVER_NAME);
		if (JSON_NODE_TYPE(node) == JSON_NODE_VALUE && json_node_get_value_type(node) == G_TYPE_STRING) {
			const gchar * name = json_node_get_string(node);
			server->parent.name = g_strdup(name);
		}
	}

	if (json_object_has_member(object, JSON_URI)) {
		JsonNode * node = json_object_get_member(object, JSON_URI);
		if (JSON_NODE_TYPE(node) == JSON_NODE_VALUE && json_node_get_value_type(node) == G_TYPE_STRING) {
			const gchar * uri = json_node_get_string(node);
			server->parent.uri = g_strdup(uri);
		}
	}

	return SERVER(server);
}
