#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "server.h"
#include "citrix-server.h"
#include "rdp-server.h"
#include "uccs-server.h"

static void server_class_init (ServerClass *klass);
static void server_init       (Server *self);
static void server_dispose    (GObject *object);
static void server_finalize   (GObject *object);

G_DEFINE_TYPE (Server, server, G_TYPE_OBJECT);

static void
server_class_init (ServerClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	object_class->dispose = server_dispose;
	object_class->finalize = server_finalize;

	return;
}

static void
server_init (Server *self)
{
	self->name = NULL;
	self->uri = NULL;
	self->last_used = FALSE;

	return;
}

static void
server_dispose (GObject *object)
{

	G_OBJECT_CLASS (server_parent_class)->dispose (object);
	return;
}

static void
server_finalize (GObject *object)
{
	Server * server = SERVER(object);

	g_free(server->name);
	g_free(server->uri);

	G_OBJECT_CLASS (server_parent_class)->finalize (object);
	return;
}

GVariant *
server_get_variant (Server * server)
{
	/* Okay, this doesn't do anything useful, but it will generate an error
	   which could be a good thing */
	g_return_val_if_fail(IS_SERVER(server), NULL);

	ServerClass * klass = SERVER_GET_CLASS(server);
	if (klass->get_properties != NULL) {
		GVariantBuilder tuple;
		g_variant_builder_init(&tuple, G_VARIANT_TYPE_TUPLE);

		if (IS_CITRIX_SERVER(server)) {
			g_variant_builder_add_value(&tuple, g_variant_new_string("ica"));
		} else if (IS_RDP_SERVER(server)) {
			g_variant_builder_add_value(&tuple, g_variant_new_string("rdp"));
		} else if (IS_UCCS_SERVER(server)) {
			g_variant_builder_add_value(&tuple, g_variant_new_string("uccs"));
		} else {
			g_assert_not_reached();
		}

		if (server->name != NULL) {
			g_variant_builder_add_value(&tuple, g_variant_new_string(server->name));
		} else {
			g_warning("Server has no name");
			g_variant_builder_add_value(&tuple, g_variant_new_string(""));
		}

		if (server->uri != NULL) {
			g_variant_builder_add_value(&tuple, g_variant_new_string(server->uri));
		} else {
			g_warning("Server has no URI");
			g_variant_builder_add_value(&tuple, g_variant_new_string(""));
		}

		g_variant_builder_add_value(&tuple, g_variant_new_boolean(server->last_used));

		GVariant * props = klass->get_properties(server);
		g_variant_builder_add_value(&tuple, props);

		if (klass->get_applications != NULL) {
			GVariant * array = klass->get_applications(server);
			g_variant_builder_add_value(&tuple, array);
		} else {
			/* NULL array of applications */
			g_variant_builder_add_value(&tuple, g_variant_new_array(G_VARIANT_TYPE("(si)"), NULL, 0));
		}

		return g_variant_builder_end(&tuple);
	}

	return NULL;
}
